from datetime import datetime
import logging

from ipware import get_client_ip

from django.conf import settings


logger = logging.getLogger(__name__)


class AuditLogMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def _get_ip_address(self):
        client_ip, is_routable = get_client_ip(self.request)
        return client_ip or "Unknown"

    # Defined so it we can mock it
    def _get_utc_now(self):
        return datetime.utcnow()

    #  Check for custom user id in request.user
    def _get_custom_user_id(self):
        if getattr(
            settings, "AUDIT_LOG_USER_FIELD", None
        ) is not None:
            if hasattr(
                self.request.user,
                settings.AUDIT_LOG_USER_FIELD,
            ):
                return getattr(
                    self.request.user,
                    settings.AUDIT_LOG_USER_FIELD,
                )

        return ""

    def _get_email(self):
        if self.request.user.is_authenticated:
            return self.request.user.email

        return "anonymous"

    def _get_first_name(self):
        if self.request.user.is_authenticated:
            return self.request.user.first_name

        return ""

    def _get_last_name(self):
        if self.request.user.is_authenticated:
            return self.request.user.last_name

        return ""

    def __call__(self, request):
        self.request = request

        logger.info(
            f"Security audit log:\t"
            f"{self._get_utc_now()}\t"
            f"{request.method}\t"
            f"{request.build_absolute_uri()}\t"
            f"{self._get_ip_address()}\t"
            f"{self._get_custom_user_id()}\t"
            f"{self._get_email()}\t"
            f"{self._get_first_name()}\t"
            f"{self._get_last_name()}"
        )

        response = self.get_response(request)
        return response



from distutils.core import setup

import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(
    name="django_audit_log_middleware",
    version="0.0.5",
    packages=setuptools.find_packages(),
    author="Ross Miller",
    author_email="ross.miller@digita.trade.gov.uk",
    url="https://github.com/uktrade/django-audit-log-middleware",
    description="Simple audit logging for Django requests",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    install_requires=[
        "django>=2.2.0",
        "django-ipware>=3.0.2",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from unittest.mock import patch, MagicMock, Mock

from django_audit_log_middleware import AuditLogMiddleware
from django.test import override_settings, TestCase


class AuditLogTests(TestCase):
    def setUp(self):
        self.middleware = AuditLogMiddleware(
            get_response=MagicMock()
        )
        self.request = Mock()
        self.ip = "1.1.1.1"
        self.request.META = { 
            "REMOTE_ADDR": self.ip,
        }
        self.request.method = "GET"
        self.request.build_absolute_uri = Mock(return_value="http://www.test.com?test=1")

        self.request.user = Mock(
            first_name="Jane",
            last_name="Test",
            email="jane@test.com",
        )

    @patch('django_audit_log_middleware.AuditLogMiddleware._get_utc_now')
    def test_log_output_on_request(self, utcnow):
        utcnow.return_value = "Test"

        with self.assertLogs('django_audit_log_middleware', level='INFO') as test_logger:
            response = self.middleware(self.request)

            expected = (
                "INFO:django_audit_log_middleware:"
                f"Security audit log:\t"
                f"{utcnow.return_value}\t"
                f"{self.request.method}\t"
                f"{self.request.build_absolute_uri()}\t"
                f"{self.ip}\t"
                f"\t"
                f"{self.request.user.email}\t"
                f"{self.request.user.first_name}\t"
                f"{self.request.user.last_name}"
            )

            self.assertEqual(expected, test_logger.output[0])

    @override_settings(AUDIT_LOG_USER_FIELD="username")
    @patch('django_audit_log_middleware.AuditLogMiddleware._get_utc_now')
    def test_log_output_with_custom_user_id(self, utcnow):
        utcnow.return_value = "Test"

        self.request.user = Mock(
            username="custom id test",
            first_name="Jane",
            last_name="Test",
            email="jane@test.com",
        )

        with self.assertLogs('django_audit_log_middleware', level='INFO') as test_logger:
            response = self.middleware(self.request)

            expected = (
                "INFO:django_audit_log_middleware:"
                f"Security audit log:\t"
                f"{utcnow.return_value}\t"
                f"{self.request.method}\t"
                f"{self.request.build_absolute_uri()}\t"
                f"{self.ip}\t"
                f"{self.request.user.username}\t"
                f"{self.request.user.email}\t"
                f"{self.request.user.first_name}\t"
                f"{self.request.user.last_name}"
            )

            self.assertEqual(expected, test_logger.output[0])
